/**
 *	Copyright (C) 2016 Docuteam GmbH
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 3
 *	as published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.docuteam.darc.ingest;

import java.util.*;

import ch.docuteam.darc.mets.Document;
import ch.docuteam.darc.mets.structmap.NodeAbstract;
import ch.docuteam.darc.mets.structmap.NodeAbstract.SubmitStatus;
import ch.docuteam.tools.out.Logger;

/**
 * 
 * 
 * @author Andreas Nef, Docuteam GmbH
 *
 */
public class AIPCreatorExtended implements AIPCreator {
	@Override
	public void initialize(Properties properties) throws Exception {
	}

	@Override
	public List<String> checkSubmission(Document document) {
		List<String> rejectMessages = new Vector<String>();
		String sipName = document.getSIPName();

		Logger.info("Performing submit-check in SIP: '" + sipName + "'");

		// Is this document read-only? If yes, reject:
		if (document.isReadOnly()) {
			Logger.debug("Can't submit because SIP is read-only: '" + sipName + "'");
			rejectMessages.add("MessageSubmitSIPIsReadOnly '" + sipName + "'");
		}

		// Is this document locked? If yes, reject:
		if (document.isLocked()) {
			Logger.debug("Can't submit because SIP is locked: '" + sipName + "'");
			rejectMessages.add("MessageSubmitSIPIsLocked '" + sipName + "'");
		}

		// Is there at least one node with submitRequest or submitUndefined? If
		// no, reject:
		boolean hasNodesThatCanBeSubmitted = false;
		for (NodeAbstract n : document.getStructureMap().getRoot().getWithDescendants())
			if (n.getSubmitStatus() == SubmitStatus.SubmitRequested
					|| n.getSubmitStatus() == SubmitStatus.SubmitUndefined) {
				hasNodesThatCanBeSubmitted = true;
				break;
			}
		if (!hasNodesThatCanBeSubmitted) {
			Logger.debug("Can't submit because SIP has no submit requests: '" + sipName + "'");
			rejectMessages.add("MessageSubmitSIPHasNoSubmitRequests '" + sipName + "'");
		}

		// Check each node in tree:
		Logger.debug("Checking all nodes in SIP: '" + sipName + "'");
		for (String message : this.submitCheckRecursive(document.getStructureMap().getRoot()))
			rejectMessages.add(message);

		if (rejectMessages.isEmpty())
			Logger.debug("SIP is OK");
		else
			Logger.debug("Can't submit SIP because: " + rejectMessages);

		return rejectMessages;
	}

	@Override
	public List<String> submit(Document document) throws Exception {
		return new ArrayList<String>(Arrays.asList("method not implemented"));
	}

	@Override
	public List<String> generateAIP(Document document) throws Exception {
		return new ArrayList<String>(Arrays.asList("method not implemented"));
	}

	@Override
	public void confirmStorage(Document document) throws Exception {
		return;
	}

	private List<String> submitCheckRecursive(NodeAbstract node) {
		List<String> rejectMessages = new Vector<String>();

		// Check each subnode if it could be submitted:
		for (NodeAbstract n : node.getWithDescendants())
			for (String message : this.submitCheck(n))
				rejectMessages.add(message);

		return rejectMessages;
	}

	private List<String> submitCheck(NodeAbstract node) {
		List<String> rejectMessages = new Vector<String>();
		String pathString = node.getPathString();

		Logger.info("Performing submit-check for node: '" + pathString + "' (" + node.getAdmId() + ")");

		if (!node.fileExists())
			rejectMessages.add("MessageSubmitNodeFileDoesNotExist '" + pathString + "'");
		if (!node.canRead())
			rejectMessages.add("MessageSubmitNodeFileIsNotReadable '" + pathString + "'");
		if (!node.canWrite())
			rejectMessages.add("MessageSubmitNodeFileIsNotWritable '" + pathString + "'");

		// Is SubmitStatus = "SubmitRequestPending"? If yes, reject (SIP is
		// currently in an ongoing submission process):
		// Is SubmitStatus = "Submitted"? If yes, reject (SIP was already
		// submitted):
		SubmitStatus submitStatus = node.getSubmitStatus();
		if (submitStatus == SubmitStatus.SubmitRequestPending)
			rejectMessages.add("MessageSubmitNodeIsAlreadySubmitPending '" + pathString + "'");
		if (submitStatus == SubmitStatus.Submitted)
			rejectMessages.add("MessageSubmitNodeIsAlreadySubmitted '" + pathString + "'");

		// Are any mandatory MD fields not set? If yes, reject:
		if (node.hasDynamicMetadataElementInstancesWhichAreMandatoryButNotSet())
			rejectMessages.add("MessageSubmitNodeMandatoryMetadataNotSet '" + pathString + "'");

		// Test if all children have either the level "undefined" or not:
		Boolean allChildrenLevelsAreUndef = null;
		for (int i = 0; i < node.getChildCount(); i++) {
			boolean childLevelIsUndef = ((NodeAbstract) node.getChildAt(i)).getLevel().isUndefined();
			if (allChildrenLevelsAreUndef == null)
				allChildrenLevelsAreUndef = childLevelIsUndef;
			else if (allChildrenLevelsAreUndef != childLevelIsUndef) {
				rejectMessages.add("MessageSubmitNodeHasHeterogenousChildren '" + pathString + "'");
				break;
			}
		}

		if (rejectMessages.isEmpty())
			Logger.info("Node is OK");
		else
			Logger.info("Can't submit node because: " + rejectMessages);

		return rejectMessages;
	}

}
